package boltease

import (
	"errors"
	"fmt"
	"reflect"
)

type any interface{}

func (b *DB) LoadStruct(path []string, dest any) error {
	destValue := reflect.ValueOf(dest)
	if destValue.Kind().String() != "ptr" {
		return errors.New("Destination must be a pointer")
	}
	d := reflect.Indirect(destValue)
	if d.Kind().String() != "struct" {
		path, name := path[:len(path)-1], path[len(path)-1]
		return b.GetForInterface(path, name, dest)
	}

	entityType := reflect.TypeOf(dest).Elem()
	fmt.Println("EntityType:", entityType.String())
	var ret error
	for i := 0; i < entityType.NumField(); i++ {
		structFld := entityType.Field(i)
		fldName := FieldName(structFld)
		fmt.Printf("Field (%d): %s (%s)\n", i, fldName, structFld.Type)
		switch structFld.Type.Kind() {
		case reflect.String:
			var wrk string
			err := b.GetForInterface(path, fldName, &wrk)
			if err != nil {
				if ret == nil {
					ret = err
				}
			} else {
				// Set the value
				reflect.ValueOf(dest).Elem().FieldByName(structFld.Name).Set(reflect.ValueOf(wrk))
			}

		case reflect.Bool:
			var wrk bool
			err := b.GetForInterface(path, fldName, &wrk)
			if err != nil {
				if ret == nil {
					ret = err
				}
			} else {
				// Set the value
				reflect.ValueOf(dest).Elem().FieldByName(structFld.Name).Set(reflect.ValueOf(wrk))
			}

		case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
			var wrk int
			err := b.GetForInterface(path, fldName, &wrk)
			if err != nil {
				if ret == nil {
					ret = err
				}
			} else {
				// Set the value
				reflect.ValueOf(dest).Elem().FieldByName(structFld.Name).Set(reflect.ValueOf(wrk))
			}
		}

		/*
			setField := reflect.ValueOf(dest).Elem().Field(i)
			value := entityType.Field(i)
			fldName := FieldName(value)
			if value.Type.Kind() == reflect.Struct {
				b.LoadStruct(append(path, fldName), setField.Interface())
			} else {
				err := b.GetForInterface(path, fldName, &value)
				if err != nil {
					fmt.Println(err)
					return err
				}
				fmt.Println("value:", value)
			}
		*/

		/*
			if value.Type.Kind() == reflect.Struct {
				b.LoadStruct(append(path, fldName), setField.Interface())
			} else {
				v, err := b.GetForType(path, fldName, value)
				if err != nil {
					return err
				}
				setField.Set(reflect.ValueOf(v))
			}
		*/
	}
	return ret
}

func (b *DB) SaveStruct(path []string, src any) error {
	t := reflect.TypeOf(src)
	fmt.Println(t.Kind())
	if t.Kind() == reflect.Pointer {
		// Save the actual struct
		return b.SaveStruct(path, reflect.Indirect(reflect.ValueOf(src)))
	}

	fields := reflect.VisibleFields(t)
	r := reflect.ValueOf(src)
	fmt.Println()
	for _, fld := range fields {
		f := r.FieldByName(fld.Name)
		fmt.Println(f, FieldName(fld))
		/*
			if (f.Kind() == reflect.Struct || f.Kind() == reflect.Pointer) && f != src {
				fmt.Println("Recursing on Field:", f)
				err := b.SaveStruct(append(path, FieldName(fld)), f)
				if err != nil {
					return err
				}
			} else {
				if err := b.Set(path, FieldName(fld), f); err != nil {
					return err
				}
			}
		*/
	}
	return nil
}

func FieldName(fld reflect.StructField) string {
	nm := fld.Name
	tag := fld.Tag.Get("boltease")
	if tag != "" {
		nm = tag
	}
	return nm
}
func FieldIgnored(fld reflect.StructField) bool {
	return fld.Tag.Get("boltease") == "-"
}

func ReflectValueToInterface(val reflect.Value) interface{} {
	switch val.Kind() {
	case reflect.Pointer:
		return ReflectValueToInterface(reflect.Indirect(val))
	case reflect.Int, reflect.Int8, reflect.Int16, reflect.Int32, reflect.Int64:
		return val.Int()
	case reflect.Bool:
		return val.Bool()
	case reflect.String:
		return val.String()
	default:
		return val.Bytes()
	}
}
